##############################################################################
#           By
#                  ___                  _  __    _ _
#                 |   \ ___ _  _ __ _  | |/ /___| | |___ _ _
#                 | |) / _ \ || / _` | | ' </ -_) | / -_) '_|
#                 |___/\___/\_,_\__, | |_|\_\___|_|_\___|_|
#                               |___/
#                 
#           Please follow the licensing this work is under! Cheers!
##############################################################################

##############################################################################
# imports

from habanero import cn
import isbnlib
import argparse
import sys

##############################################################################
# doi

def bib_from_doi(doi):
    """Retrieves the BibTeX citation for a document with a given DOI.

    Parameters
    ----------
    doi : str
        The DOI string of the desired document.

    Returns
    -------
    bib : str
        The BibTeX citation of the given DOI.
    """
    
    try:
        doi_bib = cn.content_negotiation(ids=doi, format='bibtex')

    except:
        raise

    return doi_bib


def is_doi(url):
    """Check if a URL is a DOI URL.

    Parameters
    ----------
    url : str
        Potential DOI URL.

    Returns
    -------
    bool
        True if it is a DOI URL, False if not.
    """

    try:
        cn.content_negotiation(ids=url)
        return True

    except:
        return False

##############################################################################
# isbn

def get_isbn13(isbn_gen_str):
    """Returns a clean ISBN number if one exists in the given argument.

    Parameters
    ----------
    isbn_gen_str : str
        Potentially "dirty" string containing an ISBN number, possibly embedded in an URL.

    Returns
    -------
    isbn13 : str
        ISBN 13 number
    """

    isbn_like = isbnlib.get_isbnlike(isbn_gen_str, level='loose')[-1]  # Get ISBN from a URL or ISBN string.
    clean_isbn_like = isbnlib.clean(isbn_like)

    if isbnlib.is_isbn10(clean_isbn_like):  # Check if the ISBN is ISBN10 and changing it to ISBN13 if it is.
        isbn13 = isbnlib.to_isbn13(clean_isbn_like)
    else:
        isbn13 = clean_isbn_like

    return isbn13


def bib_from_isbn(isbn13):
    """Retrieves the BibTeX citation for a document with a given ISBN-13.

    Parameters
    ----------
    isbn : str
        The ISBN-13 number of the desired document.

    Returns
    -------
    bib : str
        The BibTeX citation of the given ISBN-13.
    """
    
    try:
        isbn_dict = isbnlib.meta(isbn13)
        isbn_bib = isbnlib.registry.bibformatters['bibtex'](isbn_dict)

    except:
        raise

    return isbn_bib


def is_isbn(url):
    """Check if the given URL has an ISBN

    Parameters
    ----------
    url : str
        URL with potential ISBN.

    Returns
    -------
    bool
        True if there is an ISBN, False if not.
    """

    return not isbnlib.notisbn(url)

##############################################################################
# cli

def get_bib(work_id):
    """Returns a BibTeX string if given a DOI or ISBN; returns None if it is not.

    Parameters
    ----------
    work_id : str
        String containing a potential work reference.

    Returns
    -------
    bib_str : str
        BibTeX string.
    """

    if is_doi(work_id):
        return bib_from_doi(work_id)

    elif is_isbn(work_id):
        return bib_from_isbn(work_id)

    else:
        return None

def print_bib(work_id, bib_str):
    """Prints BibTeX string.
    """

    if bib_str:
        print(bib_str + '\n')
    else:
        print(work_id + ' not a valid DOI or ISBN\n')


def parse_args(args):
    """Argument parser: takes IDs (DOIs or ISBNs) and returns BibTeX strings.

    Parameters
    ----------
    args : list
        List of arguments.

    Returns
    -------
    bib_dict : dict
        Dictionary of BibTeX strings.
    """

    parser = argparse.ArgumentParser('bibchk')
    parser.add_argument('IDs', nargs='+', type=str, help='DOIs or ISBNs')
    parsed = parser.parse_args(args)

    bib_dict = {}
    for ID in parsed.IDs:
        bib_dict[ID] = get_bib(ID)

    return bib_dict

##############################################################################
# main

def main():
    bib_dict = parse_args(sys.argv[1::])
    
    for ID in bib_dict.keys():
        print_bib(ID, bib_dict[ID])


if __name__ == '__main__':
    main()



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['bibchk']

package_data = \
{'': ['*']}

install_requires = \
['argparse>=1.4.0,<2.0.0', 'habanero>=1.0.0,<2.0.0', 'isbnlib>=3.10.10,<4.0.0']

setup_kwargs = {
    'name': 'bibchk',
    'version': '0.1.0',
    'description': 'Simple command line program to return the BibTeX string of a given DOI or ISBN.',
    'long_description': '# bibchk\n\nSimple program to return the BibTeX string of a given DOI(s) or ISBN(s).\n\nAn example with a DOI and DOI URL:\n\n```bash\n$ bibchk 10.1002/2016JC011857 https://doi.org/10.1002/2016JC011857\n@article{Houpert_2016,\n\tdoi = {10.1002/2016jc011857},\n\turl = {https://doi.org/10.1002%2F2016jc011857},\n\tyear = 2016,\n\tmonth = {nov},\n\tpublisher = {American Geophysical Union ({AGU})},\n\tvolume = {121},\n\tnumber = {11},\n\tpages = {8139--8171},\n\tauthor = {L. Houpert and X. Durrieu de Madron and P. Testor and A. Bosse and F. D{\\textquotesingle}Ortenzio and M. N. Bouin and D. Dausse and H. Le Goff and S. Kunesch and M. Labaste and L. Coppola and L. Mortier and P. Raimbault},\n\ttitle = {Observations of open-ocean deep convection in the northwestern Mediterranean Sea: Seasonal and interannual variability of mixing and deep water masses for the 2007-2013 Period},\n\tjournal = {Journal of Geophysical Research: Oceans}\n}\n\n@article{Houpert_2016,\n\tdoi = {10.1002/2016jc011857},\n\turl = {https://doi.org/10.1002%2F2016jc011857},\n\tyear = 2016,\n\tmonth = {nov},\n\tpublisher = {American Geophysical Union ({AGU})},\n\tvolume = {121},\n\tnumber = {11},\n\tpages = {8139--8171},\n\tauthor = {L. Houpert and X. Durrieu de Madron and P. Testor and A. Bosse and F. D{\\textquotesingle}Ortenzio and M. N. Bouin and D. Dausse and H. Le Goff and S. Kunesch and M. Labaste and L. Coppola and L. Mortier and P. Raimbault},\n\ttitle = {Observations of open-ocean deep convection in the northwestern Mediterranean Sea: Seasonal and interannual variability of mixing and deep water masses for the 2007-2013 Period},\n\tjournal = {Journal of Geophysical Research: Oceans}\n}\n```\n\nor with an ISBN:\n\n```bash\n$ bibchk 0-486-60061-0\n@book{9780486600611,\n     title = {Fundamentals Of Astrodynamics},\n    author = {Roger R. Bate and Donald D. Mueller and Jerry E. White},\n      isbn = {9780486600611},\n      year = {1971},\n publisher = {Courier Corporation}\n}\n```\n',
    'author': 'Doug Keller',
    'author_email': 'dg.kllr.jr@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/BibTheque/bibchk.git',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.9,<4.0',
}


setup(**setup_kwargs)
